@@ -10,7 +10,7 @@ from .util import (
     is_sequence,
     process_data,
     processing_decorator,
-    select_processor
+    select_function
     )
 
 
@@ -55,7 +55,7 @@ def _coerce_data(schema, data, manager, routine):
         (tuple, _coerce_tuple)
         ]
 
-    coercion_function = select_processor(schema, coercion_map)
+    coercion_function = select_function(schema, coercion_map)
     return coercion_function(schema, data, manager, routine)
 
 
@@ -73,7 +73,7 @@ def _coerce_scalar(schema, data, manager, routine):
     except AttributeError:
         return data
 
-    return coercion_method(schema, data)
+    return coercion_method(data)
 
 
 def _coerce_dict(schema, data, manager, routine):
@@ -123,7 +123,7 @@ def _coerce_list(schema, data, manager, routine):
     data_dict = dict(enumerate(data))
 
     result_dict = _coerce_dict(schema_dict, data_dict, manager, routine)
-    return [result_dict[key] for key in range(len(result_dict))]
+    return [value for key, value in sorted(result_dict.items())]
 
 
 def _coerce_ListOf(list_of, data, manager, routine):
