@@ -63,6 +63,9 @@ class RuleViolation(Error):
         self.data = data
         self.location = _key_location(key)
 
+    def __str__(self):
+        return self.message
+
 
 def _key_location(key):
     if key is None:
