@@ -37,3 +37,9 @@ class Dict:
             for key, value in items
             if value is not _NotSet
             }
+
+
+class Call:
+    def __init__(self, function, **kwargs):
+        self.function = function
+        self.kwargs = kwargs
