@@ -6,11 +6,11 @@ from functools import wraps
 from .exceptions import SchemaError
 
 
-def select_processor(schema, processor_map):
-    """ Select `processor` function from `processor_map`, depending on `schema`
+def select_function(schema, function_map):
+    """ Select function from `function_map`, depending on `schema`
         type. """
 
-    for schema_type, processor in processor_map:
+    for schema_type, processor in function_map:
         if isinstance(schema, schema_type):
             return processor
 
