@@ -61,14 +61,14 @@ class TestCoerceData(unittest.TestCase):
     """ `_coerce_data` calls the appropriate coercion function depending on
         `schema` type. """
 
-    @patch('jsonmanager.coercion_tools.select_processor')
-    def test_calls_coercion_function(self, mock_select_processor):
-        """ `select_processor` called to get coercion function.
+    @patch('jsonmanager.coercion_tools.select_function')
+    def test_calls_coercion_function(self, mock_select_function):
+        """ `select_function` called to get coercion function.
             `process_data` called with coercion function, schema, data.
 
             `process_data` return value returned. """
         coercion_function = MagicMock()
-        mock_select_processor.return_value = coercion_function
+        mock_select_function.return_value = coercion_function
         coercion_function.return_value = sentinel.expected
 
         result = _coerce_data(
@@ -143,7 +143,7 @@ class TestCoerceScalar(unittest.TestCase):
 
         result = _coerce_scalar(SomeType, sentinel.data, manager, routine)
 
-        coercion_method.assert_called_with(SomeType, sentinel.data)
+        coercion_method.assert_called_with(sentinel.data)
         assert result is sentinel.expected
 
     def test_input_yes_coercion(self):
@@ -443,7 +443,7 @@ class IntegrationTest(unittest.TestCase):
 
         class Manager(CoercionManagerBase):
             # pylint: disable=unused-argument
-            def coerce_input_ExpectedType(self, schema, data):
+            def coerce_input_ExpectedType(self, data):
                 return data
         self.manager = Manager()
 
