@@ -19,12 +19,12 @@ from jsonmanager.util import(
     process_data,
     processing_decorator,
     resolve_decorator_schemas,
-    select_processor
+    select_function
     )
 
 
-class TestSelectProcessor(unittest.TestCase):
-    """ `select_processor` function. """
+class TestSelectFunctionFunctional(unittest.TestCase):
+    """ `util.select_function` """
 
     def setUp(self):
         class TargetType:
@@ -36,36 +36,36 @@ class TestSelectProcessor(unittest.TestCase):
         self.TargetType = TargetType
         self.OtherType = OtherType
 
-    def success_test(self, processor_map):
-        """ `target_processor` is returned. """
-        schema = MagicMock(self.TargetType)
-        result = select_processor(schema, processor_map)
-        assert result is sentinel.target_processor
+    def success_test(self, function_map):
+        """ `target_function` is returned. """
+        schema = self.TargetType()
+        result = select_function(schema, function_map)
+        assert result is sentinel.target_function
 
     def test_target_before_other(self):
-        """ `TargetType` before `OtherType` in `processor_map`. """
-        processor_map = [
-            (self.TargetType, sentinel.target_processor),
-            (self.OtherType, sentinel.other_processor)
+        """ `TargetType` before `OtherType` in `function_map`. """
+        function_map = [
+            (self.TargetType, sentinel.target_function),
+            (self.OtherType, sentinel.other_function),
             ]
-        self.success_test(processor_map)
+        self.success_test(function_map)
 
     def test_target_after_other(self):
-        """ `TargetType` after `OtherType` in `processor_map`. """
-        processor_map = [
-            (self.TargetType, sentinel.target_processor),
-            (self.OtherType, sentinel.other_processor)
+        """ `TargetType` after `OtherType` in `function_map`. """
+        function_map = [
+            (self.OtherType, sentinel.other_function),
+            (self.TargetType, sentinel.target_function),
             ]
-        self.success_test(processor_map)
+        self.success_test(function_map)
 
-    def test_target_type_not_in_map(self):
-        """ `TargetType` not in `processor_map`. """
-        processor_map = [
-            (self.OtherType, sentinel.other_processor)
+    def test_target_type_not_in_map_raises(self):
+        """ `TargetType` not in `function_map`. """
+        function_map = [
+            (self.OtherType, sentinel.other_function)
             ]
         schema = MagicMock(self.TargetType)
         with self.assertRaises(SchemaError):
-            select_processor(schema, processor_map)
+            select_function(schema, function_map)
 
 
 class TestIsMapping(unittest.TestCase):
